#!/bin/bash
# Licensed to the Apache Software Foundation (ASF) under one or more
# contributor license agreements.  See the NOTICE file distributed with
# this work for additional information regarding copyright ownership.
# The ASF licenses this file to You under the Apache License, Version 2.0
# (the "License"); you may not use this file except in compliance with
# the License.  You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
# -------------------------------------------------------------------------------------
#
# Ranger PolicyMigration Setup Script
#
START=$(date +%s)
action=$1
action=`echo $action | tr '[:upper:]' '[:lower:]'`

# Print log statements
function log {
  echo "$(date): $@"
}
exported_policies_json_file=$2
exported_services_json_file=$3

source env.sh

LOGFILE=${LOGFILE}

ranger_policymigration_transform_output_file=${RANGER_POLICYMIGRATION_TRANSFORM_OUTPUT_FILE}
ranger_policymigration_max_heap_size=${RANGER_POLICYMIGRATION_MAX_HEAP_SIZE}

unix_user=${UNIX_USER}
unix_user_pwd=${UNIX_USER_PWD}
unix_group=${UNIX_GROUP}

source_destination_ranger_service_type_mapping=${SOURCE_DESTINATION_RANGER_SERVICE_TYPE_MAPPING}
source_destination_ranger_service_name_mapping=${SOURCE_DESTINATION_RANGER_SERVICE_NAME_MAPPING}
source_destination_ranger_zone_name_mapping=${SOURCE_DESTINATION_RANGER_ZONE_NAME_MAPPING}
s3_bucket_name=${S3_BUCKET_NAME}
adls_storage_account=${ADLS_STORAGE_ACCOUNT}
adls_storage_account_container=${ADLS_STORAGE_ACCOUNT_CONTAINER}
service_name_for_native_policies=${SERVICE_NAME_FOR_NATIVE_POLICIES}

check_ret_status(){
    if [ $1 -ne 0 ]; then
        log "[E] $2";
        exit 1;
    fi
}

check_ret_status_for_groupadd(){
# 9 is the response if the group exists
    if [ $1 -ne 0 ] && [ $1 -ne 9 ]; then
        log "[E] $2";
        exit 1;
    fi
}

check_user_pwd(){
    if [ -z "$1" ]; then
        log "[E] The unix user password is empty. Please set user password.";
        exit 1;
    fi
}

is_command () {
    log "[I] check if command $1 exists"
    type "$1" >/dev/null
}

get_distro(){
    log "[I] Checking distribution name.."
    ver=$(cat /etc/*{issues,release,version} 2> /dev/null)
    if [[ $(echo $ver | grep DISTRIB_ID) ]]; then
        DIST_NAME=$(lsb_release -si)
    else
        DIST_NAME=$(echo $ver | cut -d ' ' -f 1 | sort -u | head -1)
    fi
    export $DIST_NAME
    log "[I] Found distribution : $DIST_NAME"

}
#Get Properties from File without erroring out if property is not there
#$1 -> propertyName $2 -> fileName $3 -> variableName $4 -> failIfNotFound
getPropertyFromFileNoExit(){
    validateProperty=$(sed '/^\#/d' $2 | grep "^$1\s*="  | tail -n 1) # for validation
    if  test -z "$validateProperty" ; then 
        log "[E] '$1' not found in $2 file while getting....!!";
        if [ $4 == "true" ] ; then
            exit 1;
        else
            value=""
        fi
    else
        value=$(echo $validateProperty | cut -d "=" -f2-)
    fi
    eval $3="'$value'"
}
#Get Properties from File
#$1 -> propertyName $2 -> fileName $3 -> variableName
getPropertyFromFile(){
    validateProperty=$(sed '/^\#/d' $2 | grep "^$1\s*="  | tail -n 1) # for validation
    if  test -z "$validateProperty" ; then log "[E] '$1' not found in $2 file while getting....!!"; exit 1; fi
    value=$(echo $validateProperty | cut -d "=" -f2-)
    eval $3="'$value'"
}

#Update Properties to File
#$1 -> propertyName $2 -> newPropertyValue $3 -> fileName
updatePropertyToFilePy(){
    $PYTHON_COMMAND_INVOKER update_property.py $1 "${2}" $3
    check_ret_status $? "Update property failed for: " $1
}

init_variables(){
    curDt=`date '+%Y%m%d%H%M%S'`
    POLICYMIGRATION_INSTALL_DIR=$PWD
}


check_java_version() {
    #Check for JAVA_HOME
    if [ "${JAVA_HOME}" == "" ]
    then
        log "[E] JAVA_HOME environment property not defined, aborting installation."
        exit 1
    fi

    export JAVA_ORACLE='Java(TM) SE Runtime Environment'
    export JAVA_BIN=${JAVA_HOME}/bin/java

    if is_command ${JAVA_BIN} ; then
        log "[I] '${JAVA_BIN}' command found"
    else
        log "[E] '${JAVA_BIN}' command not found"
        exit 1;
    fi

    version=$("$JAVA_BIN" -version 2>&1 | awk -F '"' '/version/ {print $2}')
    major=`echo ${version} | cut -d. -f1`
    minor=`echo ${version} | cut -d. -f2`
    current_java_version="$major.$minor"
    num_current_java_version=`echo $current_java_version|awk ' { printf("%3.2f\n", $0); } '`
    JAVA_VERSION_REQUIRED=`echo $JAVA_VERSION_REQUIRED | awk '{gsub(/ /,"")}1'`
    JAVA_VERSION_REQUIRED=`echo $JAVA_VERSION_REQUIRED | awk '{gsub(/'"'"'/,"")}1'`
    num_required_java_version=`echo $JAVA_VERSION_REQUIRED|awk ' { printf("%3.2f\n", $0); } '`
    if [ `echo "$num_current_java_version < $num_required_java_version" | bc` -eq 1 ];then
        log "[E] The java version must be greater than or equal to $JAVA_VERSION_REQUIRED, the current java version is $version"
        exit 1;
    fi
}

create_rollback_point() {
    DATE=`date`
    BAK_FILE=$APP-$VERSION.$DATE.bak
    log "Creating backup file : $BAK_FILE"
    cp "$APP" "$BAK_FILE"
}

update_properties() {
    newPropertyValue=''
    echo "export JAVA_HOME=${JAVA_HOME}" > ${POLICYMIGRATION_INSTALL_DIR}/conf/java_home.sh
    chmod a+rx ${POLICYMIGRATION_INSTALL_DIR}/conf/java_home.sh

    to_file_ranger=$POLICYMIGRATION_INSTALL_DIR/conf/ranger-policymigration-site.xml
    if test -f $to_file_ranger; then
        log "[I] $to_file_ranger file found"
    else
        log "[E] $to_file_ranger does not exists" ; exit 1;
    fi

    propertyName=ranger.policymigration.transform.output_file
    newPropertyValue="${ranger_policymigration_transform_output_file}"
    updatePropertyToFilePy $propertyName "${newPropertyValue}" $to_file_ranger

    propertyName=ranger.policymigration.service.type.mapping
    newPropertyValue="${source_destination_ranger_service_type_mapping}"
    updatePropertyToFilePy $propertyName "${newPropertyValue}" $to_file_ranger

    propertyName=ranger.policymigration.service.name.mapping
    newPropertyValue="${source_destination_ranger_service_name_mapping}"
    updatePropertyToFilePy $propertyName "${newPropertyValue}" $to_file_ranger

    propertyName=ranger.policymigration.zone.name.mapping
    newPropertyValue="${source_destination_ranger_zone_name_mapping}"
    updatePropertyToFilePy $propertyName "${newPropertyValue}" $to_file_ranger
   
    propertyName=ranger.policymigration.s3.bucket.name
    newPropertyValue="${s3_bucket_name}"
    updatePropertyToFilePy $propertyName "${newPropertyValue}" $to_file_ranger

    propertyName=ranger.policymigration.adls.storage.account
    newPropertyValue="${adls_storage_account}"
    updatePropertyToFilePy $propertyName "${newPropertyValue}" $to_file_ranger

    propertyName=ranger.policymigration.adls.storage.container
    newPropertyValue="${adls_storage_account_container}"
    updatePropertyToFilePy $propertyName "${newPropertyValue}" $to_file_ranger

    propertyName=ranger.policymigration.nativepolicies.service.name
    newPropertyValue="${service_name_for_native_policies}"
    updatePropertyToFilePy $propertyName "${newPropertyValue}" $to_file_ranger

    if [ "${RANGER_POLICYMIGRATION_USERS_MAPPING_FILE}" != "" ]
    then
        if ! test -f "$RANGER_POLICYMIGRATION_USERS_MAPPING_FILE"; then
            log "[E] Please provide valid file path in config RANGER_POLICYMIGRATION_USERS_MAPPING_FILE"
            exit 1
        fi
    fi

    propertyName=ranger.policymigration.users.mapping.file
    newPropertyValue="${RANGER_POLICYMIGRATION_USERS_MAPPING_FILE}"
    updatePropertyToFilePy $propertyName "${newPropertyValue}" $to_file_ranger

    if [ "${RANGER_POLICYMIGRATION_RESOURCE_MAPPING_FILE}" != "" ]
    then
        if ! test -f "$RANGER_POLICYMIGRATION_RESOURCE_MAPPING_FILE"; then
            log "[E] Please provide valid file path in config RANGER_POLICYMIGRATION_RESOURCE_MAPPING_FILE"
            exit 1
        fi
    fi

    propertyName=ranger.policymigration.resource.mapping.file
    newPropertyValue="${RANGER_POLICYMIGRATION_RESOURCE_MAPPING_FILE}"
    updatePropertyToFilePy $propertyName "${newPropertyValue}" $to_file_ranger

    if [ "${RANGER_POLICYMIGRATION_HIVE_URL_MAPPING_FILE}" != "" ]
    then
        if ! test -f "$RANGER_POLICYMIGRATION_HIVE_URL_MAPPING_FILE"; then
            log "[E] Please provide valid file path in config RANGER_POLICYMIGRATION_HIVE_URL_MAPPING_FILE"
            exit 1
        fi
    fi

    propertyName=ranger.policymigration.hive.url.resource.mapping.file
    newPropertyValue="${RANGER_POLICYMIGRATION_HIVE_URL_MAPPING_FILE}"
    updatePropertyToFilePy $propertyName "${newPropertyValue}" $to_file_ranger

    propertyName=ranger.policymigration.exclude.service.types
    newPropertyValue="${RANGER_POLICYMIGRATION_EXCLUDE_SERVICE_TYPES}"
    updatePropertyToFilePy $propertyName "${newPropertyValue}" $to_file_ranger

    propertyName=ranger.policymigration.nativepolicies.policy.name.prefix
    newPropertyValue="${POLICY_NAME_PREFIX_FOR_NATIVE_POLICIES}"
    updatePropertyToFilePy $propertyName "${newPropertyValue}" $to_file_ranger

    propertyName=ranger.policymigration.nativepolicies.policy.description
    newPropertyValue="${POLICY_DESCRIPTION_FOR_NATIVE_POLICIES}"
    updatePropertyToFilePy $propertyName "${newPropertyValue}" $to_file_ranger

}

#=====================================================================
setup_unix_user_group(){
    log "[I] Setting up UNIX user : ${unix_user} and group: ${unix_group}"

    #create group if it does not exist
    egrep "^$unix_group" /etc/group >& /dev/null
    if [ $? -ne 0 ]
    then
        groupadd ${unix_group}
        check_ret_status_for_groupadd $? "Creating group ${unix_group} failed"
    fi

    #create user if it does not exists
    id -u ${unix_user} > /dev/null 2>&1
    if [ $? -ne 0 ]
    then
        check_user_pwd ${unix_user_pwd}
        log "[I] Creating new user and adding to group"
        useradd ${unix_user} -g ${unix_group} -m
        check_ret_status $? "useradd ${unix_user} failed"

        passwdtmpfile=passwd.tmp
        if [  -f "$passwdtmpfile" ]; then
            rm -rf  ${passwdtmpfile}
        fi
        cat> ${passwdtmpfile} << EOF
${unix_user}:${unix_user_pwd}
EOF
        chpasswd <  ${passwdtmpfile}
        rm -rf  ${passwdtmpfile}
    else
        useringroup=`id ${unix_user}`
        useringrouparr=(${useringroup// / })
        if [[  ${useringrouparr[1]} =~ "(${unix_group})" ]]
        then
            log "[I] the ${unix_user} user already exists and belongs to group ${unix_group}"
        else
            log "[I] User already exists, adding it to group ${unix_group}"
            usermod -g ${unix_group} ${unix_user}
        fi
    fi
    log "[I] Setting up UNIX user : ${unix_user} and group: ${unix_group} DONE"
}

setup_install_files(){
    log "[I] Setting up installation files and directory";
    
    if [ -d ${POLICYMIGRATION_INSTALL_DIR}/lib ]; then
        chown -R ${unix_user} ${POLICYMIGRATION_INSTALL_DIR}/lib
    fi
    
    if [ -d ${POLICYMIGRATION_INSTALL_DIR}/dist ]; then
        chown -R ${unix_user} ${POLICYMIGRATION_INSTALL_DIR}/dist
    fi
    
    if [ -z "${RANGER_POLICYMIGRATION_LOG_DIR}" ] || [ ${RANGER_POLICYMIGRATION_LOG_DIR} == ${POLICYMIGRATION_INSTALL_DIR} ]; then
        RANGER_POLICYMIGRATION_LOG_DIR=${POLICYMIGRATION_INSTALL_DIR}/logs;
    fi

    if [ ! -d ${RANGER_POLICYMIGRATION_LOG_DIR} ]; then
        log "[I] ${RANGER_POLICYMIGRATION_LOG_DIR} Ranger policymigration Log folder"
        mkdir -p ${RANGER_POLICYMIGRATION_LOG_DIR}
    fi

    if [ -d ${RANGER_POLICYMIGRATION_LOG_DIR} ]; then
        chown -R ${unix_user} ${RANGER_POLICYMIGRATION_LOG_DIR}
    fi

    echo "export RANGER_POLICYMIGRATION_LOG_DIR=${RANGER_POLICYMIGRATION_LOG_DIR}" > ${POLICYMIGRATION_INSTALL_DIR}/conf/ranger-policymigration-env-logdir.sh
    chmod a+rx ${POLICYMIGRATION_INSTALL_DIR}/conf/ranger-policymigration-env-logdir.sh

}

log " --------- Running Ranger PolicyMigration Setup Script --------- "
log "[I] uname=`uname`"
log "[I] hostname=`hostname`"
init_variables
get_distro
check_java_version
setup_unix_user_group
setup_install_files
if [ "$?" == "0" ]
then
    update_properties
else
    log "[E] File config setup failed!."
    exit 1
fi

ARG_serviceExportLocation=''
if [ "${action}" != "" ]
then
    log " --------- Running Ranger PolicyMigration Import/Export Script --------- "

    for custom_env_script in `find ${POLICYMIGRATION_INSTALL_DIR}/conf/ -name "ranger-policymigration-env*"`; do
        if [ -f $custom_env_script ]; then
            . $custom_env_script
        fi
    done

    if test -f "${exported_policies_json_file}"; then
        log "[I] ${exported_policies_json_file} file found"
    else
        log "[E] ${exported_policies_json_file} does not exists" ; exit 1;
    fi

    if test -f "${exported_services_json_file}"; then
        log "[I] ${exported_services_json_file} file found"
        ARG_serviceExportLocation="-service ${exported_services_json_file}"
    fi

    JAVA_OPTS=" ${JAVA_OPTS} -XX:MetaspaceSize=100m -XX:MaxMetaspaceSize=200m -Xmx${ranger_policymigration_max_heap_size} -Xms1g -Xloggc:${POLICYMIGRATION_INSTALL_DIR}/logs/gc-worker.log -verbose:gc -XX:+UseGCLogFileRotation -XX:NumberOfGCLogFiles=10 -XX:GCLogFileSize=1m -XX:+PrintGCDetails -XX:+PrintHeapAtGC -XX:+PrintGCDateStamps"
    if [[ ${JAVA_OPTS} != *"-Duser.timezone"* ]] ;then  export JAVA_OPTS=" ${JAVA_OPTS} -Duser.timezone=UTC" ;fi

    if [ "$JAVA_HOME" != "" ]; then
        export PATH=$JAVA_HOME/bin:$PATH
    fi

    ${JAVA_HOME}/bin/java ${JAVA_OPTS} -Dlog4j.debug=true -Dlogdir=${RANGER_POLICYMIGRATION_LOG_DIR} -Dlog4j.configuration=file:${POLICYMIGRATION_INSTALL_DIR}/conf/log4j.properties -cp "${POLICYMIGRATION_INSTALL_DIR}/conf:${POLICYMIGRATION_INSTALL_DIR}/dist/*:${POLICYMIGRATION_INSTALL_DIR}/lib/*:${JAVA_HOME}/lib/*" org.apache.ranger.policymigration.RangerExportedPolicyTransformer --command ${action}  --confdir ${POLICYMIGRATION_INSTALL_DIR}/conf -policy ${exported_policies_json_file} ${ARG_serviceExportLocation} 
    if [ "$?" == "0" ]
    then
        echo "${action} of Ranger policies completed."
    else
        echo "${action} of Ranger policies failed. Please refer ${RANGER_POLICYMIGRATION_LOG_DIR}/${LOGFILE} logs for more details. "
        exit 1
    fi

    END=$(date +%s)
    DIFF=$(( $END - $START ))
    echo "Time taken: $DIFF seconds"
fi
